@@ -27,6 +27,10 @@
 #include <unistd.h>
 #endif
 #include <time.h>
+#ifdef HAVE_UNAME_SYSCALL
+#include <sys/utsname.h>
+#endif
+#include <zlib.h>
 
 #include "shared/optparser.h"
 #include "shared/misc.h"
@@ -35,6 +39,7 @@
 #include "libclamav/clamav.h"
 #include "libclamav/others.h"
 #include "libclamav/bytecode.h"
+#include "target.h"
 
 #ifndef _WIN32
 extern const struct clam_option *clam_options;
@@ -54,7 +59,8 @@ static struct _cfgfile {
 
 static const char *dbnames[] = { "main.cvd", "main.cld", "daily.cvd",
 				 "daily.cld", "safebrowsing.cvd",
-				 "safebrowsing.cld", NULL };
+				 "safebrowsing.cld", "bytecode.cvd",
+				 "bytecode.cld", NULL };
 
 static void printopts(struct optstruct *opts, int nondef)
 {
@@ -205,10 +211,85 @@ static void help(void)
     return;
 }
 
+static void print_platform(void)
+{
+#ifdef HAVE_UNAME_SYSCALL
+    struct utsname name;
+#endif
+    printf("\nPlatform information\n--------------------\n");
+#ifdef HAVE_UNAME_SYSCALL
+    uname(&name);
+    printf("uname: %s %s %s %s\n", name.sysname, name.release, name.version,
+	   name.machine);
+#endif
+    printf("OS: "TARGET_OS_TYPE", ARCH: "TARGET_ARCH_TYPE", CPU: "TARGET_CPU_TYPE"\n");
+
+#ifdef C_LINUX
+    if (!access("/usr/bin/lsb_release", X_OK)) {
+	fputs("Full OS version: ", stdout);
+	fflush(stdout);
+	if (system("/usr/bin/lsb_release -d -s") == -1) {
+	   perror("failed to determine");
+	}
+    }
+#elif defined(_WIN32)
+    {
+	/* get just some basic information, since getting the full version
+	 * is too complicated */
+	uint32_t dwVersion = GetVersion();
+	printf("Full OS version: %d.%d (%d)\n",
+	   dwVersion&0xff, (dwVersion>>8)&0xff,
+	   dwVersion<0x80000000 ? dwVersion>>16 : 0);
+    }
+#else
+    /* e.g. Solaris */
+    if (!access("/etc/release", R_OK)) {
+	char buf[1024];
+	FILE *f = fopen("/etc/release", "r");
+	fgets(buf, sizeof(buf), f);
+	printf("Full OS version: %s", buf);
+    }
+#endif
+
+#ifdef ZLIB_VERNUM
+    printf("zlib version: %s (%s), compile flags: %02lx\n",
+	   ZLIB_VERSION, zlibVersion(), zlibCompileFlags());
+#else
+    /* old zlib w/o zlibCompileFlags() */
+    printf("zlib version: %s (%s)\n",
+	   ZLIB_VERSION, zlibVersion());
+#endif
+}
+
+static void print_build(void)
+{
+    printf("\nBuild information\n-----------------\n");
+    /* Try to print information about some commonly used compilers */
+#ifdef __GNUC__
+    printf("GNU C: %s (%u.%u.%u)\n", __VERSION__, __GNUC__, __GNUC_MINOR__,
+	   __GNUC_PATCHLEVEL__);
+#endif
+#ifdef __INTEL_COMPILER
+    printf("Intel Compiler %u\n", __INTEL_COMPILER);
+#endif
+#ifdef _MSC_VER
+    printf("Microsoft Visual C++ %u\n", _MSC_VER);
+#endif
+#ifdef __SUNPRO_C
+    printf("Sun studio %u\n", __SUNPRO_C);
+#endif
+    cli_printcxxver();
+#if defined(BUILD_CPPFLAGS) && defined(BUILD_CFLAGS) && defined(BUILD_CXXFLAGS) && defined(BUILD_LDFLAGS) && defined(BUILD_CONFIGURE_FLAGS)
+    printf("CPPFLAGS: %s\nCFLAGS: %s\nCXXFLAGS: %s\nLDFLAGS: %s\nConfigure: %s\n",
+	   BUILD_CPPFLAGS, BUILD_CFLAGS, BUILD_CXXFLAGS, BUILD_LDFLAGS,
+	   BUILD_CONFIGURE_FLAGS);
+#endif
+}
+
 int main(int argc, char **argv)
 {
 	const char *dir;
-	char path[512], dbdir[512], *pt;
+	char path[512], dbdir[512], clamd_dbdir[512], *pt;
 	struct optstruct *opts, *toolopts;
 	const struct optstruct *opt;
 	unsigned int i, j;
@@ -262,6 +343,10 @@ int main(int argc, char **argv)
 	    opt = optget(toolopts, "DatabaseDirectory");
 	    strncpy(dbdir, opt->strarg, sizeof(dbdir));
 	    dbdir[sizeof(dbdir) - 1] = 0;
+	} else if(cfgfile[i].tool == OPT_CLAMD) {
+	    opt = optget(toolopts, "DatabaseDirectory");
+	    strncpy(clamd_dbdir, opt->strarg, sizeof(clamd_dbdir));
+	    clamd_dbdir[sizeof(clamd_dbdir) - 1] = 0;
 	}
 	optfree(toolopts);
     }
@@ -311,6 +396,8 @@ int main(int argc, char **argv)
 	dbdir[sizeof(dbdir) - 1] = 0;
     }
     printf("Database directory: %s\n", dbdir);
+    if(strcmp(dbdir, clamd_dbdir))
+	printf("WARNING: freshclam.conf and clamd.conf point to different database directories\n");
     flevel = cl_retflevel();
     for(i = 0; dbnames[i]; i++) {
 	snprintf(path, sizeof(path), "%s"PATHSEP"%s", dbdir, dbnames[i]);
@@ -328,5 +415,7 @@ int main(int argc, char **argv)
 	    }
 	}
     }
+    print_platform();
+    print_build();
     return 0;
 }
