@@ -78,6 +78,8 @@
 #include "libclamav/str.h"
 #include "libclamav/cvd.h"
 
+extern char updtmpdir[512];
+
 #define CHDIR_ERR(x)				\
 	if(chdir(x) == -1)			\
 	    logg("!Can't chdir to %s\n", x);
@@ -544,11 +546,6 @@ int submitstats(const char *clamdcfg, const struct optstruct *opts)
 	unsigned int qcnt, entries, submitted = 0, permfail = 0, port = 0;
 
 
-    if(optget(opts, "HTTPUserAgent")->enabled) {
-	logg("!SubmitDetectionStats: HTTPUserAgent must be disabled for SubmitDetectionStats to work\n");
-	return 56;
-    }
-
     if((opt = optget(opts, "DetectionStatsCountry"))->enabled) {
 	if(strlen(opt->strarg) != 2 || !isalpha(opt->strarg[0]) || !isalpha(opt->strarg[1])) {
 	    logg("!SubmitDetectionStats: DetectionStatsCountry requires a two-letter country code\n");
@@ -625,7 +622,10 @@ int submitstats(const char *clamdcfg, const struct optstruct *opts)
 	strncpy(newstatsdat, line, sizeof(newstatsdat));
     }
 
-    snprintf(uastr, sizeof(uastr), PACKAGE"/%s (OS: "TARGET_OS_TYPE", ARCH: "TARGET_ARCH_TYPE", CPU: "TARGET_CPU_TYPE"):%s:%s", get_version(), country ? country : "", hostid ? hostid : "");
+    if((opt = optget(opts, "HTTPUserAgent"))->enabled)
+        strncpy(uastr, opt->strarg, sizeof(uastr));
+    else
+	snprintf(uastr, sizeof(uastr), PACKAGE"/%s (OS: "TARGET_OS_TYPE", ARCH: "TARGET_ARCH_TYPE", CPU: "TARGET_CPU_TYPE"):%s:%s", get_version(), country ? country : "", hostid ? hostid : "");
     uastr[sizeof(uastr) - 1] = 0;
 
     if((opt = optget(opts, "HTTPProxyServer"))->enabled) {
@@ -673,6 +673,7 @@ int submitstats(const char *clamdcfg, const struct optstruct *opts)
 	*pt = 0;
 	pt += 4;
 
+	tms.tm_isdst = -1;
 	if(!strptime(buff, "%a %b  %d %H:%M:%S %Y", &tms) || (epoch = mktime(&tms)) == -1) {
 	    logg("!SubmitDetectionStats: Failed to convert date string\n");
 	    ret = 1;
@@ -709,12 +710,12 @@ int submitstats(const char *clamdcfg, const struct optstruct *opts)
 	    query[sizeof(query) - 1] = 0;
 	    if(mdprintf(sd,
 		"POST http://stats.clamav.net/submit.php HTTP/1.0\r\n"
-		"Host: stats.clamav.net\r\n%s"
+		"Host: stats.clamav.net\r\n%s%s%s%s"
 		"Content-Type: application/x-www-form-urlencoded\r\n"
 		"User-Agent: %s\r\n"
 		"Content-Length: %u\r\n\r\n"
 		"%s",
-		auth ? auth : "", uastr, (unsigned int) strlen(query), query) < 0)
+		auth ? auth : "", hostid ? "X-HostID: " : "", hostid ? hostid : "", hostid ? "\r\n" : "", uastr, (unsigned int) strlen(query), query) < 0)
 	    {
 		logg("!SubmitDetectionStats: Can't write to socket\n");
 		ret = 52;
@@ -1468,14 +1469,15 @@ static int updatedb(const char *dbname, const char *hostname, char *ip, int *sig
 {
 	struct cl_cvd *current, *remote;
 	const struct optstruct *opt;
-	unsigned int nodb = 0, currver = 0, newver = 0, port = 0, i, j;
+	unsigned int nodb = 0, currver = 0, newver = 0, port = 0, i, j, newsigs = 0;
 	int ret, ims = -1;
-	char *pt, cvdfile[32], localname[32], *tmpdir = NULL, *newfile, newdb[32], cwd[512];
+	char *pt, cvdfile[32], localname[32], *tmpdir = NULL, *newfile, *newfile2, newdb[32];
 	char extradbinfo[64], *extradnsreply = NULL;
 	const char *proxy = NULL, *user = NULL, *pass = NULL, *uas = NULL;
 	unsigned int flevel = cl_retflevel(), remote_flevel = 0, maxattempts;
 	unsigned int can_whitelist = 0;
 	int ctimeout, rtimeout;
+	struct cl_engine *engine;
 
 
     snprintf(cvdfile, sizeof(cvdfile), "%s.cvd", dbname);
@@ -1653,12 +1655,7 @@ static int updatedb(const char *dbname, const char *hostname, char *ip, int *sig
     if(!optget(opts, "ScriptedUpdates")->enabled)
 	nodb = 1;
 
-    if(!getcwd(cwd, sizeof(cwd))) {
-	logg("!updatedb: Can't get path of current working directory\n");
-	return 50; /* FIXME */
-    }
-    newfile = cli_gentemp(cwd);
-
+    newfile = cli_gentemp(updtmpdir);
     if(nodb) {
 	ret = getcvd(cvdfile, newfile, hostname, ip, localip, proxy, port, user, pass, uas, newver, ctimeout, rtimeout, mdat, logerr, can_whitelist);
 	if(ret) {
@@ -1671,7 +1668,7 @@ static int updatedb(const char *dbname, const char *hostname, char *ip, int *sig
     } else {
 	ret = 0;
 
-	tmpdir = cli_gentemp(".");
+	tmpdir = cli_gentemp(updtmpdir);
 	maxattempts = optget(opts, "MaxAttempts")->numarg;
 	for(i = currver + 1; i <= newver; i++) {
 	    for(j = 0; j < maxattempts; j++) {
@@ -1723,6 +1720,51 @@ static int updatedb(const char *dbname, const char *hostname, char *ip, int *sig
 	return 55; /* FIXME */
     }
 
+    if(optget(opts, "TestDatabases")->enabled && strlen(newfile) > 4) {
+	if(!(engine = cl_engine_new())) {
+	    unlink(newfile);
+	    free(newfile);
+	    return 55;
+	}
+	newfile2 = strdup(newfile);
+	if(!newfile2) {
+	    unlink(newfile);
+	    free(newfile);
+	    cl_engine_free(engine);
+	    return 55;
+	}
+	newfile2[strlen(newfile2) - 4] = '.';
+	newfile2[strlen(newfile2) - 3] = 'c';
+	newfile2[strlen(newfile2) - 2] = strstr(newdb, ".cld") ? 'l' : 'v';
+	newfile2[strlen(newfile2) - 1] = 'd';
+	if(rename(newfile, newfile2) == -1) {
+	    logg("!Can't rename %s to %s: %s\n", newfile, newfile2, strerror(errno));
+	    unlink(newfile);
+	    free(newfile);
+	    free(newfile2);
+	    cl_engine_free(engine);
+	    return 57;
+	}
+	free(newfile);
+	newfile = newfile2;
+	if((ret = cl_load(newfile, engine, &newsigs, CL_DB_PHISHING | CL_DB_PHISHING_URLS | CL_DB_BYTECODE | CL_DB_PUA)) != CL_SUCCESS) {
+	    logg("!Failed to load new database: %s\n", cl_strerror(ret));
+	    unlink(newfile);
+	    free(newfile);
+	    cl_engine_free(engine);
+	    return 55;
+	}
+	if((ret = cli_bytecode_prepare(&engine->bcs, engine->dconf->bytecode/*FIXME: dconf has no sense here*/))) {
+	    logg("!Failed to compile/load bytecode: %s\n", cl_strerror(ret));
+	    unlink(newfile);
+	    free(newfile);
+	    cl_engine_free(engine);
+	    return 55;
+	}
+	logg("*Properly loaded %u signatures from new %s\n", newsigs, newdb);
+	cl_engine_free(engine);
+    }
+
 #ifdef _WIN32
     if(!access(newdb, R_OK) && unlink(newdb)) {
 	logg("!Can't unlink %s. Please fix the problem manually and try again.\n", newdb);
@@ -1778,6 +1820,17 @@ int downloadmanager(const struct optstruct *opts, const char *hostname, const ch
 	const char *dnsdbinfo;
 #endif
 
+    pt = cli_gentemp(dbdir);
+    if(!pt)
+	return 57;
+    strncpy(updtmpdir, pt, sizeof(updtmpdir));
+    free(pt);
+    if(mkdir(updtmpdir, 0755)) {
+	logg("!Can't create temporary directory %s\n", updtmpdir);
+	logg("Hint: The database directory must be writable for UID %d or GID %d\n", getuid(), getgid());
+	return 57;
+    }
+
     time(&currtime);
     logg("ClamAV update process started at %s", ctime(&currtime));
 #ifdef HAVE_GETADDRINFO
@@ -1866,6 +1919,7 @@ int downloadmanager(const struct optstruct *opts, const char *hostname, const ch
 	    free(newver);
 
 	mirman_write("mirrors.dat", &mdat);
+	cli_rmdirs(updtmpdir);
 	return ret;
 
     } else if(ret == 0)
@@ -1880,6 +1934,7 @@ int downloadmanager(const struct optstruct *opts, const char *hostname, const ch
 	    free(newver);
 
 	mirman_write("mirrors.dat", &mdat);
+	cli_rmdirs(updtmpdir);
 	return ret;
 
     } else if(ret == 0)
@@ -1908,6 +1963,7 @@ int downloadmanager(const struct optstruct *opts, const char *hostname, const ch
 	    free(newver);
 
 	mirman_write("mirrors.dat", &mdat);
+	cli_rmdirs(updtmpdir);
 	return ret;
     } else if(ret == 0)
 	updated = 1;
@@ -1934,6 +1990,7 @@ int downloadmanager(const struct optstruct *opts, const char *hostname, const ch
 	    free(newver);
 
 	mirman_write("mirrors.dat", &mdat);
+	cli_rmdirs(updtmpdir);
 	return ret;
     } else if(ret == 0)
 	updated = 1;
@@ -1947,6 +2004,7 @@ int downloadmanager(const struct optstruct *opts, const char *hostname, const ch
 		if(newver)
 		    free(newver);
 		mirman_write("mirrors.dat", &mdat);
+		cli_rmdirs(updtmpdir);
 		return ret;
 	    } else if(ret == 0)
 		updated = 1;
@@ -1996,6 +2054,7 @@ int downloadmanager(const struct optstruct *opts, const char *hostname, const ch
 		    free(cmd);
 		    if(newver)
 			free(newver);
+		    cli_rmdirs(updtmpdir);
 		    return 75;
 		}
 
@@ -2018,5 +2077,6 @@ int downloadmanager(const struct optstruct *opts, const char *hostname, const ch
     if(newver)
 	free(newver);
 
+    cli_rmdirs(updtmpdir);
     return 0;
 }
