@@ -34,6 +34,7 @@
 #include "bytecode_priv.h"
 #include "readdb.h"
 #include "scanners.h"
+#include "bytecode_api.h"
 #include "bytecode_api_impl.h"
 #include <string.h>
 
@@ -48,6 +49,24 @@ static const uint32_t nomatch[64] = {
     0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0
 };
+static const uint32_t nooffsets[64] = {
+    CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE,
+    CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE,
+    CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE,
+    CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE,
+    CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE,
+    CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE,
+    CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE,
+    CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE,
+    CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE,
+    CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE,
+    CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE,
+    CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE,
+    CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE,
+    CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE,
+    CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE,
+    CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE, CLI_OFF_NONE
+};
 
 static const uint16_t nokind;
 static const uint32_t nofilesize;
@@ -60,16 +79,20 @@ static void context_safe(struct cli_bc_ctx *ctx)
 	ctx->hooks.kind = &nokind;
     if (!ctx->hooks.match_counts)
 	ctx->hooks.match_counts = nomatch;
+    if (!ctx->hooks.match_offsets)
+	ctx->hooks.match_counts = nooffsets;
     if (!ctx->hooks.filesize)
 	ctx->hooks.filesize = &nofilesize;
     if (!ctx->hooks.pedata)
 	ctx->hooks.pedata = &nopedata;
 }
 
+static int cli_bytecode_context_reset(struct cli_bc_ctx *ctx);
 struct cli_bc_ctx *cli_bytecode_context_alloc(void)
 {
     struct cli_bc_ctx *ctx = cli_calloc(1, sizeof(*ctx));
     ctx->bytecode_timeout = 60000;
+    cli_bytecode_context_reset(ctx);
     return ctx;
 }
 
@@ -104,9 +127,13 @@ static int cli_bytecode_context_reset(struct cli_bc_ctx *ctx)
     ctx->operands = NULL;
 
     if (ctx->outfd) {
+	cli_ctx *cctx = ctx->ctx;
 	cli_bcapi_extract_new(ctx, -1);
 	if (ctx->outfd)
 	    close(ctx->outfd);
+	if (ctx->tempfile && (!cctx || !cctx->engine->keeptmp)) {
+	    cli_unlink(ctx->tempfile);
+	}
 	free(ctx->tempfile);
 	ctx->tempfile = NULL;
 	ctx->outfd = 0;
@@ -173,11 +200,20 @@ static int cli_bytecode_context_reset(struct cli_bc_ctx *ctx)
     ctx->jsnorms = NULL;
     ctx->njsnorms = 0;
     ctx->jsnormdir = NULL;
+
+    for (i=0;i<ctx->nmaps;i++)
+	cli_bcapi_map_done(ctx, i);
+    free(ctx->maps);
+    ctx->maps = NULL;
+    ctx->nmaps = 0;
+
+    ctx->containertype = CL_TYPE_ANY;
     return CL_SUCCESS;
 }
 
 int cli_bytecode_context_clear(struct cli_bc_ctx *ctx)
 {
+    cli_ctx *cctx = (cli_ctx*)ctx->ctx;
     cli_bytecode_context_reset(ctx);
     memset(ctx, 0, sizeof(*ctx));
     return CL_SUCCESS;
@@ -418,7 +454,7 @@ static inline operand_t readOperand(struct cli_bc_func *func, unsigned char *p,
     return v;
 }
 
-static inline unsigned char *readData(const unsigned char *p, unsigned *off, unsigned len, char *ok, unsigned *datalen)
+static inline char *readData(const unsigned char *p, unsigned *off, unsigned len, char *ok, unsigned *datalen)
 {
     unsigned char *dat, *q;
     unsigned l, newoff, i;
@@ -458,13 +494,13 @@ static inline unsigned char *readData(const unsigned char *p, unsigned *off, uns
     }
     *off = newoff;
     *datalen = l;
-    return dat;
+    return (char*)dat;
 }
 
 static inline char *readString(const unsigned char *p, unsigned *off, unsigned len, char *ok)
 {
     unsigned stringlen;
-    char *str = (char*)readData(p, off, len, ok, &stringlen);
+    char *str = readData(p, off, len, ok, &stringlen);
     if (*ok && stringlen && str[stringlen-1] != '\0') {
 	str[stringlen-1] = '\0';
 	cli_errmsg("bytecode: string missing \\0 terminator: %s\n", str);
@@ -482,19 +518,23 @@ static int parseHeader(struct cli_bc *bc, unsigned char *buffer, unsigned *linel
     char ok = 1;
     unsigned offset, len, flevel;
     char *pos;
+
     if (strncmp((const char*)buffer, BC_HEADER, sizeof(BC_HEADER)-1)) {
 	cli_errmsg("Missing file magic in bytecode");
 	return CL_EMALFDB;
     }
     offset = sizeof(BC_HEADER)-1;
     len = strlen((const char*)buffer);
-    flevel = readNumber(buffer, &offset, len, &ok);
+    bc->metadata.formatlevel = readNumber(buffer, &offset, len, &ok);
     if (!ok) {
-	cli_errmsg("Unable to parse functionality level in bytecode header\n");
+	cli_errmsg("Unable to parse (format) functionality level in bytecode header\n");
 	return CL_EMALFDB;
     }
-    if (flevel != BC_FUNC_LEVEL) {
-	cli_dbgmsg("Skipping bytecode with functionality level: %u (current %u)\n", flevel, BC_FUNC_LEVEL);
+    /* we support 2 bytecode formats */
+    if (bc->metadata.formatlevel != BC_FORMAT_096 &&
+	bc->metadata.formatlevel != BC_FORMAT_LEVEL) {
+	cli_dbgmsg("Skipping bytecode with (format) functionality level: %u (current %u)\n", 
+		   bc->metadata.formatlevel, BC_FORMAT_LEVEL);
 	return CL_BREAK;
     }
     /* Optimistic parsing, check for error only at the end.*/
@@ -502,9 +542,22 @@ static int parseHeader(struct cli_bc *bc, unsigned char *buffer, unsigned *linel
     bc->metadata.sigmaker = readString(buffer, &offset, len, &ok);
     bc->metadata.targetExclude = readNumber(buffer, &offset, len, &ok);
     bc->kind = readNumber(buffer, &offset, len, &ok);
-    bc->metadata.maxStack = readNumber(buffer, &offset, len, &ok);
-    bc->metadata.maxMem = readNumber(buffer, &offset, len, &ok);
-    bc->metadata.maxTime = readNumber(buffer, &offset, len, &ok);
+    bc->metadata.minfunc = readNumber(buffer, &offset, len, &ok);
+    bc->metadata.maxfunc = readNumber(buffer, &offset, len, &ok);
+    flevel = cl_retflevel();
+    /* in 0.96 these 2 fields are unused / zero, in post 0.96 these mean
+     * min/max flevel.
+     * So 0 for min/max means no min/max
+     * Note that post 0.96 bytecode/bytecode lsig needs format 7, because
+     * 0.96 doesn't check lsig functionality level.
+     */
+    if ((bc->metadata.minfunc && bc->metadata.minfunc > flevel) ||
+        (bc->metadata.maxfunc && bc->metadata.maxfunc < flevel)) {
+      cli_dbgmsg("Skipping bytecode with (engine) functionality level %u-%u (current %u)\n",
+                 bc->metadata.minfunc, bc->metadata.maxfunc, flevel);
+      return CL_BREAK;
+    }
+    bc->metadata.maxresource = readNumber(buffer, &offset, len, &ok);
     bc->metadata.compiler = readString(buffer, &offset, len, &ok);
     bc->num_types = readNumber(buffer, &offset, len, &ok);
     bc->num_func = readNumber(buffer, &offset, len, &ok);
@@ -529,7 +582,7 @@ static int parseHeader(struct cli_bc *bc, unsigned char *buffer, unsigned *linel
 	return CL_EMALFDB;
     }
     offset++;
-    *linelength = strtol(buffer+offset, &pos, 10);
+    *linelength = strtol((const char*)buffer+offset, &pos, 10);
     if (*pos != '\0') {
 	cli_errmsg("Invalid number: %s\n", buffer+offset);
 	return CL_EMALFDB;
@@ -548,7 +601,7 @@ static int parseHeader(struct cli_bc *bc, unsigned char *buffer, unsigned *linel
     return CL_SUCCESS;
 }
 
-static int parseLSig(struct cli_bc *bc, unsigned char *buffer)
+static int parseLSig(struct cli_bc *bc, char *buffer)
 {
     const char *prefix;
     char *vnames, *vend = strchr(buffer, ';');
@@ -920,7 +973,7 @@ static int parseGlobals(struct cli_bc *bc, unsigned char *buffer)
 
 static int parseMD(struct cli_bc *bc, unsigned char *buffer)
 {
-    unsigned offset = 1, len = strlen(buffer);
+    unsigned offset = 1, len = strlen((const char*)buffer);
     unsigned numMD, i, b;
     char ok = 1;
     if (buffer[0] != 'D')
@@ -1227,8 +1280,11 @@ static int parseBB(struct cli_bc *bc, unsigned func, unsigned bb, unsigned char
 			break;
 		}
 	}
-	if (inst.opcode == OP_BC_STORE)
-	    inst.type = get_optype(bcfunc, inst.u.binop[0]);
+	if (inst.opcode == OP_BC_STORE) {
+	    int16_t t = get_optype(bcfunc, inst.u.binop[0]);
+	    if (t)
+		inst.type = t;
+	}
 	if (inst.opcode == OP_BC_COPY)
 	    inst.type = get_optype(bcfunc, inst.u.binop[1]);
 	if (!ok) {
@@ -1345,7 +1401,7 @@ int cli_bytecode_load(struct cli_bc *bc, FILE *f, struct cli_dbio *dbio, int tru
 	row++;
 	switch (state) {
 	    case PARSE_BC_LSIG:
-		rc = parseLSig(bc, (unsigned char*)buffer);
+		rc = parseLSig(bc, buffer);
 		if (rc == CL_BREAK) /* skip */ {
 		    bc->state = bc_skip;
 		    free(buffer);
@@ -1481,8 +1537,10 @@ int cli_bytecode_run(const struct cli_all_bc *bcs, const struct cli_bc *bc, stru
 	inst.u.ops.funcid = ctx->funcid;
 	inst.u.ops.ops = ctx->operands;
 	inst.u.ops.opsizes = ctx->opsizes;
+	cli_dbgmsg("Bytecode: executing in interpeter mode\n");
 	return cli_vm_execute(ctx->bc, ctx, &func, &inst);
     }
+    cli_dbgmsg("Bytecode: executing in JIT mode\n");
     return cli_vm_execute_jit(bcs, ctx, &bc->funcs[ctx->funcid]);
 }
 
@@ -1795,7 +1853,8 @@ static int cli_bytecode_prepare_interpreter(struct cli_bc *bc)
 		    MAPPTR(inst->u.unaryop);
 		    break;
 		case OP_BC_GEP1:
-		    if (bcfunc->types[inst->u.binop[1]]&0x8000) {
+		    if (inst->u.three[1]&0x80000000 ||
+			bcfunc->types[inst->u.binop[1]]&0x8000) {
                       cli_errmsg("bytecode: gep1 of alloca is not allowed\n");
                       return CL_EBYTECODE;
                     }
@@ -1807,7 +1866,8 @@ static int cli_bytecode_prepare_interpreter(struct cli_bc *bc)
                     break;
 		case OP_BC_GEPZ:
 		    /*three[0] is the type*/
-		    if (bcfunc->types[inst->u.three[1]]&0x8000)
+		    if (inst->u.three[1]&0x80000000 ||
+			bcfunc->types[inst->u.three[1]]&0x8000)
 			inst->interp_op = 5*(inst->interp_op/5);
 		    else
 			inst->interp_op = 5*(inst->interp_op/5)+3;
@@ -1856,10 +1916,12 @@ static int cli_bytecode_prepare_interpreter(struct cli_bc *bc)
 
 int cli_bytecode_prepare(struct cli_all_bc *bcs, unsigned dconfmask)
 {
-    unsigned i;
+    unsigned i, interp = 0;
     int rc;
-    if (cli_bytecode_prepare_jit(bcs) == CL_SUCCESS)
+    if (cli_bytecode_prepare_jit(bcs) == CL_SUCCESS) {
+	cli_dbgmsg("Bytecode: %u bytecode prepared with JIT\n", bcs->count);
 	return CL_SUCCESS;
+    }
     for (i=0;i<bcs->count;i++) {
 	struct cli_bc *bc = &bcs->all_bcs[i];
 	if (bc->state == bc_interp || bc->state == bc_jit)
@@ -1869,9 +1931,12 @@ int cli_bytecode_prepare(struct cli_all_bc *bcs, unsigned dconfmask)
 	    continue;
 	}
 	rc = cli_bytecode_prepare_interpreter(bc);
+	interp++;
 	if (rc != CL_SUCCESS)
 	    return rc;
     }
+    cli_dbgmsg("Bytecode: %u bytecode prepared with JIT, "
+	       "%u prepared with interpreter\n", bcs->count-interp, interp);
     return CL_SUCCESS;
 }
 
@@ -1911,6 +1976,7 @@ int cli_bytecode_runlsig(cli_ctx *cctx, const struct cli_all_bc *bcs, unsigned b
     memset(&ctx, 0, sizeof(ctx));
     cli_bytecode_context_setfuncid(&ctx, bc, 0);
     ctx.hooks.match_counts = lsigcnt;
+    ctx.hooks.match_offsets = lsigsuboff;
     cli_bytecode_context_setctx(&ctx, cctx);
     cli_bytecode_context_setfile(&ctx, map);
 
@@ -1939,6 +2005,7 @@ int cli_bytecode_runhook(cli_ctx *cctx, const struct cl_engine *engine, struct c
     const unsigned *hooks = engine->hooks[id - _BC_START_HOOKS];
     unsigned i, hooks_cnt = engine->hooks_cnt[id - _BC_START_HOOKS];
     int ret;
+    unsigned executed = 0;
 
     cli_bytecode_context_setfile(ctx, map);
     cli_dbgmsg("Bytecode executing hook id %u (%u hooks)\n", id, hooks_cnt);
@@ -1952,9 +2019,10 @@ int cli_bytecode_runhook(cli_ctx *cctx, const struct cl_engine *engine, struct c
 	}
 	cli_bytecode_context_setfuncid(ctx, bc, 0);
 	ret = cli_bytecode_run(&engine->bcs, bc, ctx);
+	executed++;
 	if (ret != CL_SUCCESS) {
 	    cli_warnmsg("Bytecode failed to run: %s\n", cl_strerror(ret));
-	    return CL_SUCCESS;
+	    continue;
 	}
 	if (ctx->virname) {
 	    cli_dbgmsg("Bytecode found virus: %s\n", ctx->virname);
@@ -1969,7 +2037,7 @@ int cli_bytecode_runhook(cli_ctx *cctx, const struct cl_engine *engine, struct c
 	if (!ret) {
 	    char *tempfile;
 	    int fd = cli_bytecode_context_getresult_file(ctx, &tempfile);
-	    if (fd != -1) {
+	    if (fd && fd != -1) {
 		if (cctx && cctx->engine->keeptmp)
 		    cli_dbgmsg("Bytecode %u unpacked file saved in %s\n",
 			       bc->id, tempfile);
@@ -1980,10 +2048,10 @@ int cli_bytecode_runhook(cli_ctx *cctx, const struct cl_engine *engine, struct c
 		ret = cli_magic_scandesc(fd, cctx);
 		if (!cctx || !cctx->engine->keeptmp)
 		    if (ftruncate(fd, 0) == -1)
-			cli_dbgmsg("ftruncate failed\n");
+			cli_dbgmsg("ftruncate failed on %d\n", fd);
 		close(fd);
 		if (!cctx || !cctx->engine->keeptmp) {
-		    if (cli_unlink(tempfile))
+		    if (tempfile && cli_unlink(tempfile))
 			ret = CL_EUNLINK;
 		}
 		free(tempfile);
@@ -1999,6 +2067,10 @@ int cli_bytecode_runhook(cli_ctx *cctx, const struct cl_engine *engine, struct c
 	}
 	cli_bytecode_context_reset(ctx);
     }
+    if (executed)
+	cli_dbgmsg("Bytecode: executed %u bytecodes for this hook\n", executed);
+    else
+	cli_dbgmsg("Bytecode: no logical signature matched, no bytecode executed\n");
     return CL_CLEAN;
 }
 
@@ -2029,7 +2101,7 @@ void cli_bytecode_describe(const struct cli_bc *bc)
     }
 
     stamp = bc->metadata.timestamp;
-    printf("Bytecode format functionality level: %u\n", BC_FUNC_LEVEL);
+    printf("Bytecode format functionality level: %u\n", bc->metadata.formatlevel);
     printf("Bytecode metadata:\n\tcompiler version: %s\n",
 	   bc->metadata.compiler ? bc->metadata.compiler : "N/A");
     printf("\tcompiled on: %s",
@@ -2053,6 +2125,9 @@ void cli_bytecode_describe(const struct cli_bc *bc)
 	    printf("Unknown (type %u)", bc->kind);
 	    break;
     }
+    /* 0 means no limit */
+    printf("\tbytecode functionality level: %u - %u\n",
+	   bc->metadata.minfunc, bc->metadata.maxfunc);
     printf("\tbytecode logical signature: %s\n",
 	       bc->lsig ? bc->lsig : "<none>");
     printf("\tvirusname prefix: %s\n",
