@@ -25,6 +25,8 @@
 #include <sys/time.h>
 #endif
 #include "ClamBCModule.h"
+#include "ClamBCDiagnostics.h"
+#include "llvm/Analysis/DebugInfo.h"
 #include "llvm/ADT/DenseMap.h"
 #include "llvm/ADT/BitVector.h"
 #include "llvm/ADT/PostOrderIterator.h"
@@ -73,6 +75,7 @@
 #include <csetjmp>
 #include <new>
 #include <cerrno>
+#include <string>
 
 #include "llvm/Config/config.h"
 #if !ENABLE_THREADS
@@ -1176,6 +1179,8 @@ public:
 			    Value *Dest = Values[inst->u.binop[1]];
 			    const PointerType *PTy = cast<PointerType>(Dest->getType());
 			    Op0 = convertOperand(func, PTy->getElementType(), inst->u.binop[0]);
+			    PTy = PointerType::getUnqual(Op0->getType());
+			    Dest = Builder.CreateBitCast(Dest, PTy);
 			    Builder.CreateStore(Op0, Dest);
 			    break;
 			}
@@ -1295,7 +1300,7 @@ public:
 			    Value *Dst = convertOperand(func, inst, inst->u.three[0]);
 			    Dst = Builder.CreatePointerCast(Dst, PointerType::getUnqual(Type::getInt8Ty(Context)));
 			    Value *Src = convertOperand(func, inst, inst->u.three[1]);
-			    Src = Builder.CreatePointerCast(Dst, PointerType::getUnqual(Type::getInt8Ty(Context)));
+			    Src = Builder.CreatePointerCast(Src, PointerType::getUnqual(Type::getInt8Ty(Context)));
 			    Value *Len = convertOperand(func, Type::getInt32Ty(Context), inst->u.three[2]);
 			    CallInst *c = Builder.CreateCall4(CF->FMemmove, Dst, Src, Len,
 								ConstantInt::get(Type::getInt32Ty(Context), 1));
@@ -1308,7 +1313,7 @@ public:
 			    Value *Dst = convertOperand(func, inst, inst->u.three[0]);
 			    Dst = Builder.CreatePointerCast(Dst, PointerType::getUnqual(Type::getInt8Ty(Context)));
 			    Value *Src = convertOperand(func, inst, inst->u.three[1]);
-			    Src = Builder.CreatePointerCast(Dst, PointerType::getUnqual(Type::getInt8Ty(Context)));
+			    Src = Builder.CreatePointerCast(Src, PointerType::getUnqual(Type::getInt8Ty(Context)));
 			    Value *Len = convertOperand(func, EE->getTargetData()->getIntPtrType(Context), inst->u.three[2]);
 			    CallInst *c = Builder.CreateCall3(CF->FRealmemcmp, Dst, Src, Len);
 			    c->setTailCall(true);
@@ -1428,6 +1433,7 @@ public:
 		if (verifyFunction(*F, PrintMessageAction) == 0) {
 			DEBUG(errs() << "Generating code\n");
 			// Codegen current function as executable machine code.
+			EE->getPointerToFunction(Functions[j]);
 			void *code = EE->getPointerToFunction(F);
 			DEBUG(errs() << "Code generation finished\n");
 
@@ -1472,6 +1478,7 @@ static void addFunctionProtos(struct CommonFunctions *CF, ExecutionEngine *EE, M
 
     EE->addGlobalMapping(CF->FHandler, (void*)(intptr_t)jit_exception_handler);
     EE->InstallLazyFunctionCreator(noUnknownFunctions);
+    EE->getPointerToFunction(CF->FHandler);
 
     std::vector<const Type*> args;
     args.push_back(PointerType::getUnqual(Type::getInt8Ty(Context)));
@@ -1527,12 +1534,15 @@ static void addFunctionProtos(struct CommonFunctions *CF, ExecutionEngine *EE, M
     CF->FRealmemset = Function::Create(DummyTy, GlobalValue::ExternalLinkage,
 					     "memset", M);
     EE->addGlobalMapping(CF->FRealmemset, (void*)(intptr_t)memset);
+    EE->getPointerToFunction(CF->FRealmemset);
     CF->FRealMemmove = Function::Create(DummyTy, GlobalValue::ExternalLinkage,
 					      "memmove", M);
     EE->addGlobalMapping(CF->FRealMemmove, (void*)(intptr_t)memmove);
+    EE->getPointerToFunction(CF->FRealMemmove);
     CF->FRealmemcpy = Function::Create(DummyTy, GlobalValue::ExternalLinkage,
 					     "memcpy", M);
     EE->addGlobalMapping(CF->FRealmemcpy, (void*)(intptr_t)memcpy);
+    EE->getPointerToFunction(CF->FRealmemcpy);
 
     args.clear();
     args.push_back(PointerType::getUnqual(Type::getInt8Ty(Context)));
@@ -1542,6 +1552,7 @@ static void addFunctionProtos(struct CommonFunctions *CF, ExecutionEngine *EE, M
 				 args, false);
     CF->FRealmemcmp = Function::Create(FuncTy_5, GlobalValue::ExternalLinkage, "memcmp", M);
     EE->addGlobalMapping(CF->FRealmemcmp, (void*)(intptr_t)memcmp);
+    EE->getPointerToFunction(CF->FRealmemcmp);
 }
 
 }
@@ -1751,10 +1762,21 @@ int cli_bytecode_prepare_jit(struct cli_all_bc *bcs)
 		case 7:
 		    dest = (void*)(intptr_t)cli_apicalls7[api->idx];
 		    break;
+		case 8:
+		    dest = (void*)(intptr_t)cli_apicalls8[api->idx];
+		    break;
+		case 9:
+		    dest = (void*)(intptr_t)cli_apicalls9[api->idx];
+		    break;
 		default:
 		    llvm_unreachable("invalid api type");
 	    }
+	    if (!dest) {
+		std::string reason((Twine("No mapping for builtin api ")+api->name).str());
+		llvm_error_handler(0, reason);
+	    }
 	    EE->addGlobalMapping(F, dest);
+	    EE->getPointerToFunction(F);
 	    apiFuncs[i] = F;
 	}
 
@@ -1774,6 +1796,7 @@ int cli_bytecode_prepare_jit(struct cli_all_bc *bcs)
 	Function *SFail = Function::Create(FTy, Function::ExternalLinkage,
 					      "__stack_chk_fail", M);
 	EE->addGlobalMapping(SFail, (void*)(intptr_t)jit_ssp_handler);
+        EE->getPointerToFunction(SFail);
 
 	for (unsigned i=0;i<bcs->count;i++) {
 	    const struct cli_bc *bc = &bcs->all_bcs[i];
@@ -1991,8 +2014,10 @@ void cli_bytecode_debug_printsrc(const struct cli_bc_ctx *ctx)
 	return;
     }
     assert(ctx->line < lines->linev.size());
-    SMDiagnostic diag(ctx->file, ctx->line ? ctx->line : -1,
-		 ctx->col ? ctx->col-1 : -1,
+
+    int line = (int)ctx->line ? (int)ctx->line : -1;
+    int col = (int)ctx->col ? (int)ctx->col : -1;
+    SMDiagnostic diag(ctx->file, line, col,
 		 "", std::string(lines->linev[ctx->line-1], lines->linev[ctx->line]-1));
     diag.Print("[trace]", errs());
 }
@@ -2002,3 +2027,51 @@ void cli_bytecode_printversion()
 {
   cl::PrintVersionMessage();
 }
+
+void cli_printcxxver()
+{
+    /* Try to print information about some commonly used compilers */
+#ifdef __GNUC__
+    printf("GNU C++: %s (%u.%u.%u)\n", __VERSION__, __GNUC__, __GNUC_MINOR__,
+	   __GNUC_PATCHLEVEL__);
+#endif
+#ifdef __INTEL_COMPILER
+    printf("Intel Compiler C++ %u\n", __INTEL_COMPILER);
+#endif
+#ifdef _MSC_VER
+    printf("Microsoft Visual C++ %u\n", _MSC_VER);
+#endif
+}
+
+namespace ClamBCModule {
+void stop(const char *msg, llvm::Function* F, llvm::Instruction* I)
+{
+    llvm::errs() << msg << "\n";
+}
+}
+
+void printValue(llvm::Value *V, bool a, bool b) {
+    std::string DisplayName;
+    std::string Type;
+    unsigned Line;
+    std::string File;
+    std::string Dir;
+    if (!getLocationInfo(V, DisplayName, Type, Line, File, Dir)) {
+	errs() << *V << "\n";
+	return;
+    }
+    errs() << "'" << DisplayName << "' (" << File << ":" << Line << ")";
+}
+
+void printLocation(llvm::Instruction *I, bool a, bool b) {
+    if (MDNode *N = I->getMetadata("dbg")) {
+	DILocation Loc(N);
+	errs() << Loc.getFilename() << ":" << Loc.getLineNumber();
+	if (unsigned Col = Loc.getColumnNumber()) {
+  	    errs() << ":" << Col;
+  	}
+  	errs() << ": ";
+  	return;
+    }
+    errs() << *I << ":\n";
+}
