@@ -1,8 +1,7 @@
 /*
- *  Copyright (C) 2005 - 2006 Tomasz Kojm <tkojm@clamav.net>
+ *  Copyright (C) 2007-2009 Sourcefire, Inc.
  *
- *  Header structures based on ELF: Executable and Linkable Format, Portable
- *  Formats Specification, Version 1.1
+ *  Authors: Tomasz Kojm
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -26,6 +25,7 @@
 #include "clamav.h"
 #include "execs.h"
 #include "others.h"
+#include "fmap.h"
 
 struct elf_file_hdr32 {
     unsigned char e_ident[16];
@@ -44,6 +44,23 @@ struct elf_file_hdr32 {
     uint16_t e_shstrndx;
 };
 
+struct elf_file_hdr64 {
+    unsigned char e_ident[16];
+    uint16_t e_type;
+    uint16_t e_machine;
+    uint32_t e_version;
+    uint64_t e_entry;
+    uint64_t e_phoff;
+    uint64_t e_shoff;
+    uint32_t e_flags;
+    uint16_t e_ehsize;
+    uint16_t e_phentsize;
+    uint16_t e_phnum;
+    uint16_t e_shentsize;
+    uint16_t e_shnum;
+    uint16_t e_shstrndx;
+};
+
 struct elf_program_hdr32 {
     uint32_t p_type;
     uint32_t p_offset;
@@ -55,6 +72,17 @@ struct elf_program_hdr32 {
     uint32_t p_align;
 };
 
+struct elf_program_hdr64 {
+    uint32_t p_type;
+    uint32_t p_flags;
+    uint64_t p_offset;
+    uint64_t p_vaddr;
+    uint64_t p_paddr;
+    uint64_t p_filesz;
+    uint64_t p_memsz;
+    uint64_t p_align;
+};
+
 struct elf_section_hdr32 {
     uint32_t sh_name;
     uint32_t sh_type;
@@ -68,8 +96,21 @@ struct elf_section_hdr32 {
     uint32_t sh_entsize;
 };
 
-int cli_scanelf(int desc, cli_ctx *ctx);
+struct elf_section_hdr64 {
+    uint32_t sh_name;
+    uint32_t sh_type;
+    uint64_t sh_flags;
+    uint64_t sh_addr;
+    uint64_t sh_offset;
+    uint64_t sh_size;
+    uint32_t sh_link;
+    uint32_t sh_info;
+    uint64_t sh_addralign;
+    uint64_t sh_entsize;
+};
 
-int cli_elfheader(int desc, struct cli_exe_info *elfinfo);
+int cli_scanelf(cli_ctx *ctx);
+
+int cli_elfheader(fmap_t *map, struct cli_exe_info *elfinfo);
 
 #endif
