@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2007-2008 Sourcefire, Inc.
+ *  Copyright (C) 2007-2009 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
  *
@@ -24,20 +24,30 @@
 #include "matcher.h"
 #include "filetypes.h"
 #include "cltypes.h"
+#include "fmap.h"
+
+#define BM_BOUNDARY_EOL	1
 
 struct cli_bm_patt {
     unsigned char *pattern, *prefix;
-    char *virname, *offset;
+    char *virname;
+    uint32_t offdata[4], offset_min, offset_max;
     struct cli_bm_patt *next;
     uint16_t length, prefix_length;
     uint16_t cnt;
     unsigned char pattern0;
-    uint8_t target;
+    uint32_t boundary, filesize;
 };
 
-int cli_bm_addpatt(struct cli_matcher *root, struct cli_bm_patt *pattern);
+struct cli_bm_off {
+    uint32_t *offset, *offtab, cnt, pos;
+};
+
+int cli_bm_addpatt(struct cli_matcher *root, struct cli_bm_patt *pattern, const char *offset);
 int cli_bm_init(struct cli_matcher *root);
-int cli_bm_scanbuff(const unsigned char *buffer, uint32_t length, const char **virname, const struct cli_matcher *root, uint32_t offset, cli_file_t ftype, int fd);
+int cli_bm_initoff(const struct cli_matcher *root, struct cli_bm_off *data, fmap_t *map);
+void cli_bm_freeoff(struct cli_bm_off *data);
+int cli_bm_scanbuff(const unsigned char *buffer, uint32_t length, const char **virname, const struct cli_bm_patt **patt, const struct cli_matcher *root, uint32_t offset, fmap_t *map, struct cli_bm_off *offdata);
 void cli_bm_free(struct cli_matcher *root);
 
 #endif
