@@ -51,6 +51,11 @@
 #include "filtering.h"
 #include "perflogging.h"
 
+#ifdef HAVE__INTERNAL__SHA_COLLECT
+#include "sha256.h"
+#include "sha1.h"
+#endif
+
 #ifdef CLI_PERF_LOGGING
 
 static inline void PERF_LOG_FILTER(int32_t pos, int32_t length, int8_t trie)
@@ -387,6 +392,36 @@ int cli_checkfp(unsigned char *digest, size_t size, cli_ctx *ctx)
 	sprintf(md5 + i * 2, "%02x", digest[i]);
     md5[32] = 0;
     cli_dbgmsg("FP SIGNATURE: %s:%u:%s\n", md5, (unsigned int) size, *ctx->virname ? *ctx->virname : "Name");
+
+#ifdef HAVE__INTERNAL__SHA_COLLECT
+    if((ctx->options & CL_SCAN_INTERNAL_COLLECT_SHA) && ctx->sha_collect>0) {
+        SHA1Context sha1;
+        SHA256_CTX sha256;
+        fmap_t *map = *ctx->fmap;
+        char *ptr;
+        uint8_t shash1[SHA1_HASH_SIZE*2+1];
+        uint8_t shash256[SHA256_HASH_SIZE*2+1];
+
+        if((ptr = fmap_need_off_once(map, 0, size))) {
+            sha256_init(&sha256);
+            sha256_update(&sha256, ptr, size);
+            sha256_final(&sha256, &shash256[SHA256_HASH_SIZE]);
+            for(i=0; i<SHA256_HASH_SIZE; i++)
+                sprintf((char *)shash256+i*2, "%02x", shash256[SHA256_HASH_SIZE+i]);
+
+            SHA1Init(&sha1);
+            SHA1Update(&sha1, ptr, size);
+            SHA1Final(&sha1, &shash1[SHA1_HASH_SIZE]);
+            for(i=0; i<SHA1_HASH_SIZE; i++)
+                sprintf((char *)shash1+i*2, "%02x", shash1[SHA1_HASH_SIZE+i]);
+
+	    cli_errmsg("COLLECT:%s:%s:%u:%s:%s\n", shash256, shash1, size, *ctx->virname, ctx->entry_filename);
+        } else
+            cli_errmsg("can't compute sha\n!");
+        ctx->sha_collect = -1;
+    }
+#endif
+
     return CL_VIRUS;
 }
 
@@ -402,34 +437,92 @@ static int matchicon(cli_ctx *ctx, const char *grp1, const char *grp2)
 
 int cli_scandesc(int desc, cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli_matched_type **ftoffset, unsigned int acmode)
 {
-    int ret = CL_EMEM;
+    int ret = CL_EMEM, empty;
     fmap_t *map = *ctx->fmap;
 
-    if((*ctx->fmap = fmap(desc, 0, 0))) {
+    if((*ctx->fmap = fmap_check_empty(desc, 0, 0, &empty))) {
 	ret = cli_fmap_scandesc(ctx, ftype, ftonly, ftoffset, acmode, NULL);
 	map->dont_cache_flag = (*ctx->fmap)->dont_cache_flag;
 	funmap(*ctx->fmap);
     }
     *ctx->fmap = map;
+    if(empty)
+	return CL_CLEAN;
     return ret;
 }
 
+int cli_lsig_eval(cli_ctx *ctx, struct cli_matcher *root, struct cli_ac_data *acdata)
+{
+	unsigned int i, evalcnt;
+	uint64_t evalids;
+	int (*einfo)(fmap_t *, struct cli_exe_info *) = NULL;
+	struct cli_exe_info exeinfo;
+	fmap_t *map = *ctx->fmap;
+
+    for(i = 0; i < root->ac_lsigs; i++) {
+	evalcnt = 0;
+	evalids = 0;
+	cli_ac_chkmacro(root, acdata, i);
+	if(cli_ac_chklsig(root->ac_lsigtable[i]->logic, root->ac_lsigtable[i]->logic + strlen(root->ac_lsigtable[i]->logic), acdata->lsigcnt[i], &evalcnt, &evalids, 0) == 1) {
+	    if(root->ac_lsigtable[i]->tdb.container && root->ac_lsigtable[i]->tdb.container[0] != ctx->container_type)
+		continue;
+	    if(root->ac_lsigtable[i]->tdb.filesize && (root->ac_lsigtable[i]->tdb.filesize[0] > map->len || root->ac_lsigtable[i]->tdb.filesize[1] < map->len))
+		continue;
+
+	    if(root->ac_lsigtable[i]->tdb.ep || root->ac_lsigtable[i]->tdb.nos) {
+		einfo = NULL;
+		if(root->type == 1)
+		    einfo = cli_peheader;
+		else if(root->type == 6)
+		    einfo = cli_elfheader;
+		else if(root->type == 9)
+		    einfo = cli_machoheader;
+		if(!einfo)
+		    continue;
+		memset(&exeinfo, 0, sizeof(exeinfo));
+		if(einfo(map, &exeinfo))
+		    continue;
+		if(exeinfo.section)
+		    free(exeinfo.section);
+		if(root->ac_lsigtable[i]->tdb.ep && (root->ac_lsigtable[i]->tdb.ep[0] > exeinfo.ep || root->ac_lsigtable[i]->tdb.ep[1] < exeinfo.ep))
+		    continue;
+		if(root->ac_lsigtable[i]->tdb.nos && (root->ac_lsigtable[i]->tdb.nos[0] > exeinfo.nsections || root->ac_lsigtable[i]->tdb.nos[1] < exeinfo.nsections))
+		    continue;
+	    }
+	    if(root->ac_lsigtable[i]->tdb.icongrp1 || root->ac_lsigtable[i]->tdb.icongrp2) {
+		if(matchicon(ctx, root->ac_lsigtable[i]->tdb.icongrp1, root->ac_lsigtable[i]->tdb.icongrp2) == CL_VIRUS) {
+		    if(ctx->virname)
+			*ctx->virname = root->ac_lsigtable[i]->virname;
+		    return CL_VIRUS;
+		} else {
+		    continue;
+		}
+	    }
+	    if(!root->ac_lsigtable[i]->bc_idx) {
+		if(ctx->virname)
+		    *ctx->virname = root->ac_lsigtable[i]->virname;
+		return CL_VIRUS;
+	    }
+	    if(cli_bytecode_runlsig(ctx, &ctx->engine->bcs, root->ac_lsigtable[i]->bc_idx, ctx->virname, acdata->lsigcnt[i], acdata->lsigsuboff[i], map) == CL_VIRUS) {
+		return CL_VIRUS;
+	    }
+	}
+    }
+    return CL_CLEAN;
+}
 
 int cli_fmap_scandesc(cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli_matched_type **ftoffset, unsigned int acmode, unsigned char *refhash)
 {
  	unsigned char *buff;
 	int ret = CL_CLEAN, type = CL_CLEAN, bytes;
-	unsigned int i, evalcnt, bm_offmode = 0;
+	unsigned int i, bm_offmode = 0;
 	uint32_t maxpatlen, offset = 0;
-	uint64_t evalids;
 	struct cli_ac_data gdata, tdata;
 	struct cli_bm_off toff;
 	cli_md5_ctx md5ctx;
 	unsigned char digest[16];
 	struct cli_matcher *groot = NULL, *troot = NULL;
 	fmap_t *map = *ctx->fmap;
-	int (*einfo)(fmap_t *, struct cli_exe_info *) = NULL;
-	struct cli_exe_info exeinfo;
 
     if(!ctx->engine) {
 	cli_errmsg("cli_scandesc: engine == NULL\n");
@@ -529,60 +622,8 @@ int cli_fmap_scandesc(cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli
 	offset += bytes - maxpatlen;
     }
 
-#define LSIGEVAL(xroot, xdata) \
-    for(i = 0; i < xroot->ac_lsigs; i++) { \
-	evalcnt = 0; \
-	evalids = 0; \
-	cli_ac_chkmacro(xroot, &xdata, i);\
-	if(cli_ac_chklsig(xroot->ac_lsigtable[i]->logic, xroot->ac_lsigtable[i]->logic + strlen(xroot->ac_lsigtable[i]->logic), xdata.lsigcnt[i], &evalcnt, &evalids, 0) == 1) { \
-	    if(xroot->ac_lsigtable[i]->tdb.container && xroot->ac_lsigtable[i]->tdb.container[0] != ctx->container_type) \
-		continue; \
-	    if(xroot->ac_lsigtable[i]->tdb.filesize && (xroot->ac_lsigtable[i]->tdb.filesize[0] > map->len || xroot->ac_lsigtable[i]->tdb.filesize[1] < map->len)) \
-		continue; \
-	    \
-	    if(xroot->ac_lsigtable[i]->tdb.ep || xroot->ac_lsigtable[i]->tdb.nos) { \
-		einfo = NULL; \
-		if(xroot->type == 1) \
-		    einfo = cli_peheader; \
-		else if(xroot->type == 6) \
-		    einfo = cli_elfheader; \
-		else if(xroot->type == 9) \
-		    einfo = cli_machoheader; \
-		if(!einfo) \
-		    continue; \
-		memset(&exeinfo, 0, sizeof(exeinfo)); \
-		if(einfo(map, &exeinfo)) \
-		    continue; \
-		if(exeinfo.section) \
-		    free(exeinfo.section); \
-		if(xroot->ac_lsigtable[i]->tdb.ep && (xroot->ac_lsigtable[i]->tdb.ep[0] > exeinfo.ep || xroot->ac_lsigtable[i]->tdb.ep[1] < exeinfo.ep)) \
-		    continue; \
-		if(xroot->ac_lsigtable[i]->tdb.nos && (xroot->ac_lsigtable[i]->tdb.nos[0] > exeinfo.nsections || xroot->ac_lsigtable[i]->tdb.nos[1] < exeinfo.nsections)) \
-		    continue; \
-	    } \
-	    if(xroot->ac_lsigtable[i]->tdb.icongrp1 || xroot->ac_lsigtable[i]->tdb.icongrp2) { \
-		if(matchicon(ctx, xroot->ac_lsigtable[i]->tdb.icongrp1, xroot->ac_lsigtable[i]->tdb.icongrp2) == CL_VIRUS) { \
-		    ret = CL_VIRUS; \
-		    break; \
-		} else { \
-		    continue; \
-		} \
-	    } \
-	    if(!xroot->ac_lsigtable[i]->bc_idx) { \
-		if(ctx->virname) \
-		    *ctx->virname = xroot->ac_lsigtable[i]->virname; \
-		ret = CL_VIRUS; \
-		break; \
-	    } \
-	    if(cli_bytecode_runlsig(ctx, &ctx->engine->bcs, xroot->ac_lsigtable[i]->bc_idx, ctx->virname, xdata.lsigcnt[i], xdata.lsigsuboff[i], map) == CL_VIRUS) { \
-		ret = CL_VIRUS; \
-		break; \
-	    } \
-	} \
-    }
-
     if(troot) {
-	LSIGEVAL(troot, tdata);
+	ret = cli_lsig_eval(ctx, troot, &tdata);
 	cli_ac_freedata(&tdata);
 	if(bm_offmode)
 	    cli_bm_freeoff(&toff);
@@ -590,7 +631,7 @@ int cli_fmap_scandesc(cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli
 
     if(groot) {
 	if(ret != CL_VIRUS)
-	    LSIGEVAL(groot, gdata);
+	    ret = cli_lsig_eval(ctx, groot, &gdata);
 	cli_ac_freedata(&gdata);
     }
 
