@@ -33,30 +33,24 @@
 #endif
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifndef	C_WINDOWS
+#include <dirent.h>
+#ifndef	_WIN32
 #include <sys/wait.h>
 #include <sys/time.h>
-#include <dirent.h>
 #endif
 #include <time.h>
 #include <fcntl.h>
-#ifndef	C_WINDOWS
+#ifdef	HAVE_PWD_H
 #include <pwd.h>
 #endif
 #include <errno.h>
 #include "target.h"
-#ifndef	C_WINDOWS
-#include <sys/time.h>
-#endif
 #ifdef	HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
 #ifdef	HAVE_MALLOC_H
 #include <malloc.h>
 #endif
-#if	defined(_MSC_VER) && defined(_DEBUG)
-#include <crtdbg.h>
-#endif
 
 #include "clamav.h"
 #include "others.h"
@@ -67,10 +61,6 @@
 #include "matcher-ac.h"
 #include "md5.h"
 
-#ifndef	O_BINARY
-#define	O_BINARY	0
-#endif
-
 static unsigned char name_salt[16] = { 16, 38, 97, 12, 8, 4, 72, 196, 217, 144, 33, 124, 18, 11, 17, 253 };
 
 #ifdef CL_NOTHREADS
@@ -118,9 +108,11 @@ void cli_dbgmsg_internal(const char *str, ...)
 int cli_matchregex(const char *str, const char *regex)
 {
 	regex_t reg;
-	int match;
-
-    if(cli_regcomp(&reg, regex, REG_EXTENDED | REG_NOSUB) == 0) {
+	int match, flags = REG_EXTENDED | REG_NOSUB;
+#ifdef _WIN32
+    flags |= REG_ICASE;
+#endif
+    if(cli_regcomp(&reg, regex, flags) == 0) {
 	match = (cli_regexec(&reg, str, 0, NULL, 0) == REG_NOMATCH) ? 0 : 1;
 	cli_regfree(&reg);
 	return match;
@@ -138,11 +130,7 @@ void *cli_malloc(size_t size)
 	return NULL;
     }
 
-#if defined(_MSC_VER) && defined(_DEBUG)
-    alloc = _malloc_dbg(size, _NORMAL_BLOCK, __FILE__, __LINE__);
-#else
     alloc = malloc(size);
-#endif
 
     if(!alloc) {
 	cli_errmsg("cli_malloc(): Can't allocate memory (%lu bytes).\n", (unsigned long int) size);
@@ -161,11 +149,7 @@ void *cli_calloc(size_t nmemb, size_t size)
 	return NULL;
     }
 
-#if defined(_MSC_VER) && defined(_DEBUG)
-    alloc = _calloc_dbg(nmemb, size, _NORMAL_BLOCK, __FILE__, __LINE__);
-#else
     alloc = calloc(nmemb, size);
-#endif
 
     if(!alloc) {
 	cli_errmsg("cli_calloc(): Can't allocate memory (%lu bytes).\n", (unsigned long int) (nmemb * size));
@@ -224,11 +208,7 @@ char *cli_strdup(const char *s)
         return NULL;
     }
 
-#if defined(_MSC_VER) && defined(_DEBUG)
-    alloc = _strdup_dbg(s, _NORMAL_BLOCK, __FILE__, __LINE__);
-#else
     alloc = strdup(s);
-#endif
 
     if(!alloc) {
         cli_errmsg("cli_strdup(): Can't allocate memory (%u bytes).\n", (unsigned int) strlen(s));
@@ -352,6 +332,10 @@ int cli_writen(int fd, const void *buff, unsigned int count)
 
 int cli_filecopy(const char *src, const char *dest)
 {
+
+#ifdef _WIN32
+    return (!CopyFileA(src, dest, 0));
+#else
 	char *buffer;
 	int s, d, bytes;
 
@@ -377,7 +361,29 @@ int cli_filecopy(const char *src, const char *dest)
     close(s);
 
     return close(d);
+#endif
 }
+
+#ifndef P_tmpdir
+#ifdef _WIN32
+#define P_tmpdir "C:\\"
+#else
+#define P_tmpdir "/tmp"
+#endif /* _WIN32 */
+#endif /* P_tmpdir */
+
+const char *cli_gettmpdir(void) {
+	const char *tmpdir;
+    if(
+#ifdef _WIN32
+	!(tmpdir = getenv("TEMP")) && !(tmpdir = getenv("TMP"))
+#else
+	!(tmpdir = getenv("TMPDIR"))
+#endif
+    ) tmpdir = P_tmpdir;
+    return tmpdir;
+}
+
 struct dirent_data {
     char *filename;
     const char *dirname;
@@ -518,9 +524,11 @@ int cli_ftw(char *path, int flags, int maxdepth, cli_ftw_cb callback, struct cli
 	char *pathend;
 	/* trim slashes so that dir and dir/ behave the same when
 	 * they are symlinks, and we are not following symlinks */
-	while (path[0] == '/' && path[1] == '/') path++;
+#ifndef _WIN32
+	while (path[0] == *PATHSEP && path[1] == *PATHSEP) path++;
+#endif
 	pathend = path + strlen(path);
-	while (pathend > path && pathend[-1] == '/') --pathend;
+	while (pathend > path && pathend[-1] == *PATHSEP) --pathend;
 	*pathend = '\0';
     }
     if(pathchk && pathchk(path, data) == 1)
@@ -563,10 +571,11 @@ static int cli_ftw_dir(const char *dirname, int flags, int maxdepth, cli_ftw_cb
 
     if((dd = opendir(dirname)) != NULL) {
 	struct dirent *dent;
+	int err;
 	errno = 0;
 	ret = CL_SUCCESS;
 #ifdef HAVE_READDIR_R_3
-	while(!readdir_r(dd, &result.d, &dent) && dent) {
+	while(!(err = readdir_r(dd, &result.d, &dent)) && dent) {
 #elif defined(HAVE_READDIR_R_2)
 	while((dent = (struct dirent *) readdir_r(dd, &result.d))) {
 #else
@@ -613,10 +622,10 @@ static int cli_ftw_dir(const char *dirname, int flags, int maxdepth, cli_ftw_cb
 		if (ret != CL_SUCCESS)
 		    break;
 	    }
-            if(!strcmp(dirname, "/"))
-		sprintf(fname, "/%s", dent->d_name);
+            if(!strcmp(dirname, PATHSEP))
+		sprintf(fname, PATHSEP"%s", dent->d_name);
 	    else
-		sprintf(fname, "%s/%s", dirname, dent->d_name);
+		sprintf(fname, "%s"PATHSEP"%s", dirname, dent->d_name);
 
 	    if(pathchk && pathchk(fname, data) == 1) {
 		free(fname);
@@ -674,10 +683,22 @@ static int cli_ftw_dir(const char *dirname, int flags, int maxdepth, cli_ftw_cb
 	    }
 	    errno = 0;
 	}
+#ifndef HAVE_READDIR_R_3
+	err = errno;
+#endif
 	closedir(dd);
+	if (err) {
+	    char errs[128];
+	    cli_errmsg("Unable to readdir() directory %s: %s\n", dirname,
+		       cli_strerror(errno, errs, sizeof(errs)));
+	    /* report error to callback using error_stat */
+	    ret = callback(NULL, NULL, dirname, error_stat, data);
+	    if (ret != CL_SUCCESS)
+		return ret;
+	}
 
 	if (entries) {
-	    qsort(entries, entries_cnt, sizeof(*entries), ftw_compare);
+	    cli_qsort(entries, entries_cnt, sizeof(*entries), ftw_compare);
 	    for (i = 0; i < entries_cnt; i++) {
 		struct dirent_data *entry = &entries[i];
 		ret = handle_entry(entry, flags, maxdepth-1, callback, data, pathchk);
@@ -761,18 +782,12 @@ char *cli_gentemp(const char *dir)
         const char *mdir;
 	unsigned char salt[16 + 32];
 	int i;
+    size_t len;
 
-    if(!dir) {
-	if((mdir = getenv("TMPDIR")) == NULL)
-#ifdef P_tmpdir
-	    mdir = P_tmpdir;
-#else
-	    mdir = "/tmp";
-#endif
-    } else
-	mdir = dir;
+    mdir = dir ? dir : cli_gettmpdir();
 
-    name = (char *) cli_calloc(strlen(mdir) + 1 + 32 + 1 + 7, sizeof(char));
+    len = strlen(mdir) + 42;
+    name = (char *) cli_calloc(len, sizeof(char));
     if(!name) {
 	cli_dbgmsg("cli_gentemp('%s'): out of memory\n", mdir);
 	return NULL;
@@ -799,12 +814,7 @@ char *cli_gentemp(const char *dir)
 	return NULL;
     }
 
-#ifdef	C_WINDOWS
-	sprintf(name, "%s\\clamav-", mdir);
-#else
-	sprintf(name, "%s/clamav-", mdir);
-#endif
-    strncat(name, tmp, 32);
+	snprintf(name, len, "%s"PATHSEP"clamav-%s", mdir, tmp);
     free(tmp);
 
     return(name);
