@@ -108,6 +108,14 @@ if((ndesc = open(tempfile, O_RDWR|O_CREAT|O_TRUNC|O_BINARY, S_IRWXU)) < 0) { \
     } \
 }
 
+#ifdef HAVE__INTERNAL__SHA_COLLECT
+#define SHA_OFF do { ctx->sha_collect = -1; } while(0)
+#define SHA_RESET do { ctx->sha_collect = sha_collect; } while(0)
+#else
+#define SHA_OFF do {} while(0)
+#define SHA_RESET do {} while(0)
+#endif
+
 #define FSGCASE(NAME,FREESEC) \
     case 0: /* Unpacked and NOT rebuilt */ \
 	cli_dbgmsg(NAME": Successfully decompressed\n"); \
@@ -148,12 +156,15 @@ if((ndesc = open(tempfile, O_RDWR|O_CREAT|O_TRUNC|O_BINARY, S_IRWXU)) < 0) { \
         free(exe_sections); \
 	lseek(ndesc, 0, SEEK_SET); \
 	cli_dbgmsg("***** Scanning rebuilt PE file *****\n"); \
+	SHA_OFF; \
 	if(cli_magic_scandesc(ndesc, ctx) == CL_VIRUS) { \
 	    close(ndesc); \
 	    CLI_TMPUNLK(); \
 	    free(tempfile); \
+	    SHA_RESET; \
 	    return CL_VIRUS; \
 	} \
+	SHA_RESET; \
 	close(ndesc); \
 	CLI_TMPUNLK(); \
 	free(tempfile); \
@@ -511,7 +522,7 @@ int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
 	char *src = NULL, *dest = NULL;
 	int ndesc, ret = CL_CLEAN, upack = 0, native=0;
 	size_t fsize;
-	uint32_t valign, falign, hdr_size, j, offset;
+	uint32_t valign, falign, hdr_size, j;
 	struct cli_exe_section *exe_sections;
 	struct cli_matcher *md5_sect;
 	char timestr[32];
@@ -519,7 +530,9 @@ int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
 	struct cli_bc_ctx *bc_ctx;
 	fmap_t *map;
 	struct cli_pe_hook_data pedata;
-
+#ifdef HAVE__INTERNAL__SHA_COLLECT
+	int sha_collect = ctx->sha_collect;
+#endif
 
     if(!ctx) {
 	cli_errmsg("cli_scanpe: ctx == NULL\n");
@@ -1922,13 +1935,16 @@ int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
 	    cli_dbgmsg("UPX/FSG: Decompressed data saved in %s\n", tempfile);
 
 	cli_dbgmsg("***** Scanning decompressed file *****\n");
+	SHA_OFF;
 	if((ret = cli_magic_scandesc(ndesc, ctx)) == CL_VIRUS) {
 	    close(ndesc);
 	    CLI_TMPUNLK();
 	    free(tempfile);
+	    SHA_RESET;
 	    return CL_VIRUS;
 	}
 
+	SHA_RESET;
 	close(ndesc);
 	CLI_TMPUNLK();
 	free(tempfile);
@@ -2506,6 +2522,8 @@ int cli_peheader(fmap_t *map, struct cli_exe_info *peinfo)
 
 		    while(sfi_sz > 6) { /* enum all stringtables - RESUMABLE */
 			uint32_t st_sz = cli_readint32(vptr) & 0xffff;
+			uint8_t *next_vptr = vptr + st_sz;
+			uint32_t next_sfi_sz = sfi_sz - st_sz;
 
 			if(st_sz > sfi_sz || st_sz <= 24) {
 			    /* - the content is larger than the container
@@ -2520,33 +2538,39 @@ int cli_peheader(fmap_t *map, struct cli_exe_info *peinfo)
 			while(st_sz > 6) {  /* enum all strings - RESUMABLE */
 			    uint32_t s_sz, s_key_sz, s_val_sz;
 
-			    s_sz = s_val_sz = cli_readint32(vptr);
-			    s_sz &= 0xffff;
-			    s_val_sz = (s_val_sz & 0xffff0000)>>15;
-			    if(s_sz > st_sz || s_sz <= 6 + 2 + 2 || s_val_sz > s_sz - 6 - 2 - 2) {
+			    s_sz = (cli_readint32(vptr) & 0xffff) + 3;
+			    s_sz &= ~3;
+			    if(s_sz > st_sz || s_sz <= 6 + 2 + 8) {
 				/* - the content is larger than the container
-				 * - there's no room for a minimal string (headers(6) + key(2) + padding(2))
+				 * - there's no room for a minimal string
 				 * - there's no room for the value */
 				st_sz = 0;
 				sfi_sz = 0;
 				break; /* force a hard fail */
 			    }
 
-			    if(!s_val_sz) {
-				/* skip unset value */
+			    /* ~wcstrlen(key) */
+			    for(s_key_sz = 6; s_key_sz+1 < s_sz; s_key_sz += 2) {
+				if(vptr[s_key_sz] || vptr[s_key_sz+1]) continue;
+				s_key_sz += 2;
+				break;
+			    }
+
+			    s_key_sz += 3;
+			    s_key_sz &= ~3;
+
+			    if(s_key_sz >= s_sz) {
+				/* key overflow */
 				vptr += s_sz;
 				st_sz -= s_sz;
 				continue;
 			    }
 
-			    /* ~wcstrlen(key) */
-			    for(s_key_sz = 0; s_key_sz < s_sz - 6 - s_val_sz; s_key_sz += 2) {
-				if(vptr[6+s_key_sz] || vptr[6+s_key_sz+1]) continue;
-				s_key_sz += 2;
-				break;
-			    }
-			    if(s_key_sz >= s_sz - 6 - s_val_sz) {
-				/* key overflow */
+			    s_val_sz = s_sz - s_key_sz;
+			    s_key_sz -= 6;
+
+			    if(s_val_sz <= 2) {
+				/* skip unset value */
 				vptr += s_sz;
 				st_sz -= s_sz;
 				continue;
@@ -2565,13 +2589,11 @@ int cli_peheader(fmap_t *map, struct cli_exe_info *peinfo)
 				char *k, *v, *s;
 
 				/* FIXME: skip too long strings */
-				k = cli_utf16toascii(vptr + 6, s_key_sz);
+				k = cli_utf16toascii((const char*)vptr + 6, s_key_sz);
 				if(k) {
-				    s_key_sz += 6 + 3;
-				    s_key_sz &= ~3;
-				    v = cli_utf16toascii(vptr + s_key_sz, s_val_sz);
+				    v = cli_utf16toascii((const char*)vptr + s_key_sz + 6, s_val_sz);
 				    if(v) {
-					s = cli_str2hex(vptr + 6, s_key_sz + s_val_sz - 6);
+					s = cli_str2hex((const char*)vptr + 6, s_key_sz + s_val_sz - 6);
 					if(s) {
 					    cli_dbgmsg("VersionInfo (%x): '%s'='%s' - VI:%s\n", (uint32_t)(vptr - baseptr + 6), k, v, s);
 					    free(s);
@@ -2584,8 +2606,8 @@ int cli_peheader(fmap_t *map, struct cli_exe_info *peinfo)
 			    vptr += s_sz;
 			    st_sz -= s_sz;
 			} /* enum all strings - RESUMABLE */
-			vptr += st_sz;
-			sfi_sz -= st_sz;
+			vptr = next_vptr;
+			sfi_sz = next_sfi_sz * (sfi_sz != 0);
 		    } /* enum all stringtables - RESUMABLE */
 		    break;
 		} /* look for stringfileinfo - NOT RESUMABLE */
