@@ -999,7 +999,7 @@ static int lsigattribs(char *attribs, struct cli_lsig_tdb *tdb)
 	};
 	struct lsig_attrib *apt;
 	char *tokens[ATTRIB_TOKENS], *pt, *pt2;
-	unsigned int v1, v2, v3, i, j, tokens_count;
+	unsigned int v1, v2, v3, i, j, tokens_count, have_newext = 0;
 	uint32_t cnt, off[ATTRIB_TOKENS];
 
 
@@ -1025,6 +1025,14 @@ static int lsigattribs(char *attribs, struct cli_lsig_tdb *tdb)
 	    return 1;
 	}
 
+	if(!strcmp(apt->name, "Engine")) {
+	    if(i) {
+		cli_errmsg("lsigattribs: For backward compatibility the Engine attribute must be on the first position\n");
+		return -1;
+	    }
+	} else if(strcmp(apt->name, "Target"))
+	    have_newext = 1;
+
 	switch(apt->type) {
 	    case CLI_TDB_UINT:
 		if(!cli_isnumber(pt)) {
@@ -1141,6 +1149,10 @@ static int lsigattribs(char *attribs, struct cli_lsig_tdb *tdb)
 	}
     }
 
+    if(have_newext && (!tdb->engine || tdb->engine[0] < 51)) {
+	cli_errmsg("lsigattribs: For backward compatibility all signatures using new attributes must have the Engine attribute present and set to min_level of at least 51 (0.96)\n");
+	return -1;
+    }
     return 0;
 }
 
@@ -1366,6 +1378,7 @@ static int cli_loadldb(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 
 static int cli_loadcbc(FILE *fs, struct cl_engine *engine, unsigned int *signo, unsigned int options, struct cli_dbio *dbio, const char *dbname)
 {
+    char buf[4096];
     int rc;
     struct cli_all_bc *bcs = &engine->bcs;
     struct cli_bc *bc;
@@ -1407,6 +1420,10 @@ static int cli_loadcbc(FILE *fs, struct cl_engine *engine, unsigned int *signo,
     }
 
     rc = cli_bytecode_load(bc, fs, dbio, security_trust);
+    /* read remainder of DB, needed because cvd.c checks that we read the entire
+     * file */
+    while (cli_dbgets(buf, sizeof(buf), fs, dbio)) {}
+
     if (rc != CL_SUCCESS) {
 	cli_errmsg("Unable to load %s bytecode: %s\n", dbname, cl_strerror(rc));
 	return rc;
@@ -1419,6 +1436,7 @@ static int cli_loadcbc(FILE *fs, struct cl_engine *engine, unsigned int *signo,
     bc->id = bcs->count;/* must set after _load, since load zeroes */
     sigs++;
     if (bc->kind == BC_LOGICAL || bc->lsig) {
+        unsigned oldsigs = sigs;
 	if (!bc->lsig) {
 	    cli_errmsg("Bytecode %s has logical kind, but missing logical signature!\n", dbname);
 	    return CL_EMALFDB;
@@ -1430,6 +1448,12 @@ static int cli_loadcbc(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 		       bc->lsig, dbname, cl_strerror(rc));
 	    return rc;
 	}
+        if (sigs != oldsigs) {
+          /* compiler ensures Engine field in lsig matches the one in bytecode,
+           * so this should never happen. */
+          cli_errmsg("Bytecode logical signature skipped, but bytecode itself not?");
+          return CL_EMALFDB;
+        }
     }
     if (bc->kind != BC_LOGICAL) {
 	if (bc->lsig) {
@@ -1596,6 +1620,11 @@ static int cli_loadinfo(FILE *fs, struct cl_engine *engine, unsigned int options
 	int ret = CL_SUCCESS, dsig = 0;
 	SHA256_CTX ctx;
 
+
+    if(!dbio) {
+	cli_errmsg("cli_loadinfo: .info files can only be loaded from within database container files\n");
+	return CL_EMALFDB;
+    }
     sha256_init(&ctx);
     while(cli_dbgets(buffer, FILEBUFF, fs, dbio)) {
 	line++;
@@ -1651,7 +1680,7 @@ static int cli_loadinfo(FILE *fs, struct cl_engine *engine, unsigned int options
 	    ret = CL_EMEM;
 	    break;
 	}
-	new->name = (unsigned char *) cli_mpool_strdup(engine->mempool, tokens[0]);
+	new->name = cli_mpool_strdup(engine->mempool, tokens[0]);
 	if(!new->name) {
 	    mpool_free(engine->mempool, new);
 	    ret = CL_EMEM;
@@ -2280,10 +2309,10 @@ int cli_load(const char *filename, struct cl_engine *engine, unsigned int *signo
 	ret = cli_loaddb(fs, engine, signo, options, dbio, dbname);
 
     } else if(cli_strbcasestr(dbname, ".cvd")) {
-	ret = cli_cvdload(fs, engine, signo, options, 0, dbname);
+	ret = cli_cvdload(fs, engine, signo, options, 0, filename);
 
     } else if(cli_strbcasestr(dbname, ".cld")) {
-	ret = cli_cvdload(fs, engine, signo, options, 1, dbname);
+	ret = cli_cvdload(fs, engine, signo, options, 1, filename);
 
     } else if(cli_strbcasestr(dbname, ".hdb")) {
 	ret = cli_loadmd5(fs, engine, signo, MD5_HDB, options, dbio, dbname);
