@@ -733,7 +733,7 @@ static int cli_scanmscab(int desc, cli_ctx *ctx, off_t sfx_offset)
 	if(ctx->engine->maxscansize && ctx->scansize + ctx->engine->maxfilesize >= ctx->engine->maxscansize)
 	    file->max_size = ctx->engine->maxscansize - ctx->scansize;
 	else
-	    file->max_size = ctx->engine->maxfilesize;
+	    file->max_size = ctx->engine->maxfilesize ? ctx->engine->maxfilesize : 0xffffffff;
 
 	cli_dbgmsg("CAB: Extracting file %s to %s, size %u, max_size: %u\n", file->name, tempname, file->length, (unsigned int) file->max_size);
 	file->written_size = 0;
@@ -1023,7 +1023,7 @@ static int cli_scanscript(cli_ctx *ctx)
 	struct text_norm_state state;
 	char *tmpname = NULL;
 	int ofd = -1, ret;
-	const struct cli_matcher *troot = ctx->engine->root[7];
+	struct cli_matcher *troot = ctx->engine->root[7];
 	uint32_t maxpatlen = troot ? troot->maxpatlen : 0, offset = 0;
 	struct cli_matcher *groot = ctx->engine->root[0];
 	struct cli_ac_data gmdata, tmdata;
@@ -1045,6 +1045,7 @@ static int cli_scanscript(cli_ctx *ctx)
 			cli_dbgmsg("cli_scanscript: Can't generate temporary file/descriptor\n");
 			return ret;
 		}
+		cli_dbgmsg("cli_scanscript: saving normalized file to %s\n", tmpname);
 	}
 
 	if(!(normalized = cli_malloc(SCANBUFF + maxpatlen))) {
@@ -1096,13 +1097,18 @@ static int cli_scanscript(cli_ctx *ctx)
 		cli_dbgmsg("cli_scanscript: short read during normalizing\n");
 	    }
 	}
-	cli_ac_freedata(&tmdata);
-	cli_ac_freedata(&gmdata);
 	if(ctx->engine->keeptmp) {
 		free(tmpname);
 		close(ofd);
 	}
 	free(normalized);
+	if(ret != CL_VIRUS) {
+	    ret = cli_lsig_eval(ctx, troot, &tmdata);
+	    if(ret != CL_VIRUS)
+		ret = cli_lsig_eval(ctx, groot, &gmdata);
+	}
+	cli_ac_freedata(&tmdata);
+	cli_ac_freedata(&gmdata);
 
 	return ret;
 }
@@ -1900,6 +1906,10 @@ int cli_magic_scandesc(int desc, cli_ctx *ctx)
 	unsigned char hash[16];
 	bitset_t *old_hook_lsig_matches;
 
+#ifdef HAVE__INTERNAL__SHA_COLLECT
+    if(ctx->sha_collect>0) ctx->sha_collect = 0;
+#endif
+
     cli_dbgmsg("in cli_magic_scandesc (reclevel: %u/%u)\n", ctx->recursion, ctx->engine->maxreclevel);
     if(ctx->engine->maxreclevel && ctx->recursion > ctx->engine->maxreclevel) {
         cli_dbgmsg("cli_magic_scandesc: Archive recursion limit exceeded (%u, max: %u)\n", ctx->recursion, ctx->engine->maxreclevel);
@@ -1977,6 +1987,10 @@ int cli_magic_scandesc(int desc, cli_ctx *ctx)
 	ctx->hook_lsig_matches = old_hook_lsig_matches;
 	ret_from_magicscan(CL_EREAD);
     }
+
+#ifdef HAVE__INTERNAL__SHA_COLLECT
+    if(!ctx->sha_collect && type==CL_TYPE_MSEXE) ctx->sha_collect = 1;
+#endif
     lseek(desc, 0, SEEK_SET); /* FIXMEFMAP: remove ? */
 
     ctx->hook_lsig_matches = cli_bitset_init();
@@ -2331,6 +2345,21 @@ int cl_scandesc(int desc, const char **virname, unsigned long int *scanned, cons
 	return CL_EMEM;
     ctx.hook_lsig_matches = cli_bitset_init();
 
+#ifdef HAVE__INTERNAL__SHA_COLLECT
+    if(scanoptions & CL_SCAN_INTERNAL_COLLECT_SHA) {
+	char link[32];
+	ssize_t linksz;
+
+	snprintf(link, sizeof(link), "/proc/self/fd/%u", desc);
+	link[sizeof(link)-1]='\0';
+	if((linksz=readlink(link, ctx.entry_filename, sizeof(ctx.entry_filename)))==-1) {
+	    cli_errmsg("failed to resolve filename for descriptor %d (%s)\n", desc, link);
+	    strcpy(ctx.entry_filename, "NO_IDEA");
+	} else
+	    ctx.entry_filename[linksz]='\0';
+    } while(0);
+#endif
+
     rc = cli_magic_scandesc(desc, &ctx);
 
     cli_bitset_free(ctx.hook_lsig_matches);
