@@ -170,6 +170,15 @@ void print_version(const char *dbdir)
 
     free(path);
 }
+
+int check_flevel(void)
+{
+    if(cl_retflevel() < CL_FLEVEL) {
+	fprintf(stderr, "ERROR: This tool requires libclamav with functionality level %u or higher (current f-level: %u)\n", CL_FLEVEL, cl_retflevel());
+	return 1;
+    }
+    return 0;
+}
 #endif
 
 const char *filelist(const struct optstruct *opts, int *err)
@@ -286,7 +295,6 @@ int daemonize(void)
 #endif
 }
 
-#ifndef CL_NOLIBCLAMAV
 int match_regex(const char *filename, const char *pattern)
 {
 	regex_t reg;
@@ -310,7 +318,6 @@ int match_regex(const char *filename, const char *pattern)
 	cli_regfree(&reg);
 	return match;
 }
-#endif
 
 int cfg_tcpsock(const struct optstruct *opts, struct sockaddr_in *tcpsock, in_addr_t defaultbind)
 {
