@@ -103,6 +103,7 @@ static const struct dblist_s {
     { "main.sdb",   1 },    { "daily.sdb",  1 },
     { "main.zmd",   1 },    { "daily.zmd",  1 },
     { "main.rmd",   1 },    { "daily.rmd",  1 },
+    { "main.idb",   0 },    { "daily.idb",  0 },
     { "main.fp",    0 },    { "daily.fp",   0 },
     { "main.pdb",   1 },    { "daily.pdb",  1 },    { "safebrowsing.gdb", 1 },
     { "main.wdb",   0 },    { "daily.wdb",  0 },    { "safebrowsing.wdb", 0 },
@@ -715,6 +716,29 @@ static int build(const struct optstruct *opts)
 		}
 	    }
 	    closedir(dd);
+	    if(!access("last.hdb", R_OK)) {
+		if(!dblist2) {
+		    mprintf("!build: dblist2 == NULL (no .cbc files?)\n");
+		    return -1;
+		}
+		lspt = dblist2;
+		while(lspt->next)
+		    lspt = lspt->next;
+		lspt->next = (struct dblist_scan *) malloc(sizeof(struct dblist_scan));
+		if(!lspt->next) {
+		    FREE_LS(dblist2);
+		    mprintf("!build: Memory allocation error\n");
+		    return -1;
+		}
+		lspt->next->name = strdup("last.hdb");
+		lspt->next->next = NULL;
+		if(!lspt->next->name) {
+		    FREE_LS(dblist2);
+		    mprintf("!build: Memory allocation error\n");
+		    return -1;
+		}
+		entries += countlines("last.hdb");
+	    }
 	} else {
 	    for(i = 0; dblist[i].name; i++)
 		if(dblist[i].count && strstr(dblist[i].name, dbname) && !access(dblist[i].name, R_OK))
@@ -1464,7 +1488,7 @@ static int comparesha(const char *dbname)
 	    break;
 	}
 	if(!(sha = sha256file(tokens[0], NULL))) {
-	    mprintf("!verifydiff: Can't generate MD5 for %s\n", buff);
+	    mprintf("!verifydiff: Can't generate SHA256 for %s\n", buff);
 	    ret = -1;
 	    break;
 	}
@@ -1521,7 +1545,7 @@ static int compare(const char *oldpath, const char *newpath, FILE *diff)
 	long opos;
 
 
-    if((omd5 = cli_md5file(oldpath))) {
+    if(!access(oldpath, R_OK) && (omd5 = cli_md5file(oldpath))) {
 	if(!(nmd5 = cli_md5file(newpath))) {
 	    mprintf("!compare: Can't get MD5 checksum of %s\n", newpath);
 	    free(omd5);
@@ -2568,6 +2592,9 @@ int main(int argc, char **argv)
         struct optstruct *opts;
 	struct stat sb;
 
+    if(check_flevel())
+	exit(1);
+
     opts = optparse(NULL, argc, argv, 1, OPT_SIGTOOL, 0, NULL);
     if(!opts) {
 	mprintf("!Can't parse command line options\n");
